use std::path::Path;
use std::{fmt::Display, path::PathBuf};

use crate::error::Error;

use super::{Section, Symbol};

pub type SectionIterBox<'iter> = Box<dyn Iterator<Item = Result<Section, Error>> + 'iter>;

pub type SymbolIterBox<'iter> = Box<dyn Iterator<Item = (&'iter str, Symbol)> + 'iter>;

/// Contains all the needed getters to construct a final
/// mushed and relocated executable from an object file
pub trait Relocatable: Display + Sized {
    // + TryFrom<PathBuf, Error = Error> {
    fn new(origin: PathBuf, object_index: usize) -> Result<Self, Error>;

    fn origin(&self) -> &Path; // not same as section's path since this one's supposed to be cannonical

    fn sections(&self) -> SectionIterBox;

    fn symbols(&self) -> SymbolIterBox;

    fn bytes(&self, offset: u64, size: u64) -> Result<&[u8], Error>;
}
